#------------------------------------------------------------------
# encodargs --- convert encoding of argument files on the fly
#
# Manuel Collado, http://mcollado.z15.es
# December, 2012 - Public domain
#
# USAGE
#   gawk -f encodargs.awk -f myprogram.awk  file[::encoding] ...
#
# TODO
#   Get the gawk internal encoding from the gawk core, instead of guessing
#   Convert this stuff into an extension shared library

BEGIN {
   # guess the internal gawk encoding
   # the following code emulates lines 887-895 of regcomp.c
   #--------
   #  codeset_name = getenv ("LC_ALL");
   #  if (codeset_name == NULL || codeset_name[0] == '\0')
   #    codeset_name = getenv ("LC_CTYPE");
   #  if (codeset_name == NULL || codeset_name[0] == '\0')
   #    codeset_name = getenv ("LANG");
   #  if (codeset_name == NULL)
   #    codeset_name = "";
   #  else if (strchr (codeset_name, '.') !=  NULL)
   #    codeset_name = strchr (codeset_name, '.') + 1;
   #--------
   # get current locale
   if ("LC_ALL" in ENVIRON) _encoding = ENVIRON["LC_ALL"]
   else if ("LC_CTYPE" in ENVIRON) _encoding = ENVIRON["LC_CTYPE"]
   else if ("LANG" in ENVIRON) _encoding = ENVIRON["LANG"]
   else _encoding = ""
   if (!sub(/^.*\./, "", _encoding)) {  # should be a trailing .encoding
      _encoding = "CP1252" # assume Windows western european codepage
   }
   AWKENCODING = toupper(_encoding)

   # replace remote arguments with local copies
   for (_k = 1; _k < ARGC; _k++) {
      _arg = ARGV[_k]
      if (split(_arg, _filenc, /\:\:/) == 2) {
         _encodarg = "encodarg~" _k
         _file = _filenc[1]
         _encoding = toupper(_filenc[2])
         if (_encoding == AWKENCODING) {  # same encoding, just drop the trailing part
            ARGV[_k]= _file
         } else {   # let iconv do the conversion to a temporary file
            printf("encodargs: reencoding: %s(%s) as %s(%s)\n", _file, _encoding, _encodarg, AWKENCODING) > "/dev/stderr"
            _command = "iconv -f " _encoding " -t " AWKENCODING " " _file " > " _encodarg
            system(_command)
            _filename[_encodarg] = _file
            ARGV[_k]= _encodarg
         }
      }
   }
}

# restore original argument names when processing files
FNR==1 && (FILENAME in _filename) {
   FILENAME = _filename[FILENAME]
}

